import Numeric
import Data.List
import qualified Text.ParserCombinators.Parsec as P
import qualified Text.ParserCombinators.Parsec.Token as T
import qualified Text.ParserCombinators.Parsec.Language as L

lexer      = T.makeTokenParser
   (L.emptyDef { L.commentLine = "//",
                 L.identStart  = P.letter,
                 L.identLetter = P.choice [P.char '-', P.char '\'', P.alphaNum] })

whiteSpace = T.whiteSpace lexer
symbol     = T.symbol lexer
identifier = T.identifier lexer
brackets   = T.brackets lexer
braces     = T.braces lexer
natural    = T.natural lexer

data Value = Nat Integer | Flo Double | Table [(String, Value)] 
  deriving (Eq, Ord)

-- Parse JS table
table = whiteSpace >> braces (P.sepBy entry (P.char ','))
entry = do
  whiteSpace
  key <- (identifier P.<|> (show `fmap` natural))
  P.char ':'
  val <- value
  return (key, val)
value = whiteSpace >> (Nat `fmap` natural P.<|> Table `fmap` table)

-- Rename key strings to match our conventions
rename (Nat n) = Nat n
rename (Flo f) = Flo f
rename (Table s) = Table (map (\ (k, v) -> (renameKey k, rename v)) s)

renameKey k = case lookup k renameTable of
  Just k' -> k'
  Nothing -> k

renameTable = 
  [ ("1", "Warrior")
  , ("2", "Paladin")
  , ("3", "Hunter")
  , ("4", "Rogue")
  , ("5", "Priest")
  , ("6", "Death Knight")
  , ("7", "Shaman")
  , ("8", "Mage")
  , ("9", "Warlock")
  , ("11", "Druid")
  , ("agi", "AGI")
  , ("arcsplpwr", "ARCANE_SPELL_DMG")
  , ("armor", "ARMOR")
  , ("armorpenrtng", "ARMOR_PENETRATION_RATING")
  , ("atkpwr", "AP")
  , ("block", "BLOCK_VALUE")
  , ("blockrtng", "BLOCK_RATING")
  , ("critstrkrtng", "MELEE_CRIT_RATING")
  , ("defrtng", "DEFENSE_RATING")
  , ("dodgertng", "DODGE_RATING")
  , ("exprtng", "EXPERTISE_RATING")
  , ("feratkpwr", "FERAL_AP")
  , ("firsplpwr", "FIRE_SPELL_DMG")
  , ("frosplpwr", "FROST_SPELL_DMG")
  , ("hastertng", "MELEE_HASTE_RATING")
  , ("hitrtng", "MELEE_HIT_RATING")
  , ("int", "INT")
  , ("manargn", "MANA_REG")
  , ("mledps", "DPS")
  , ("parryrtng", "PARRY_RATING")
  , ("rgddps", "DPS")
  , ("shasplpwr", "SHADOW_SPELL_DMG")
  , ("spi", "SPI")
  , ("splpwr", "SPELL_DMG")
  , ("sta", "STA")
  , ("str", "STR") ]

-- Normalize values to 0-25 range
norm' m (Nat n) = Flo (25 * fromIntegral n / fromIntegral m)
norm' m (Flo f) = Flo (25 * f / fromIntegral m)
norm' m (Table s) = Table (map (\ (k, v) -> (k, norm' m' v)) s)
  where
    m' = maximum $ map ( \ (k, v) -> case v of
      Flo f -> ceiling f
      Nat n -> n
      _     -> m) s
norm x = norm' 25 x

-- Output LUA table
instance Show Value where
  show (Flo f) = showFFloat (Just 1) f ""
  show (Nat n) = show n
  show (Table s) = 
    "{" ++ 
    concat (intersperse ", " (map (\ (k, v) -> "[\"" ++ k ++ "\"] = " ++ show v) s)) ++
    "}"

main = do
  s <- getContents
  let e_x = P.parse value "" s
  case e_x of
    Left e -> print e
    Right x -> do
      putStr "Considerater.wowheadDefaults = "
      print (norm (rename x))
  return ()
